package com.bpcl.lpg.core.strategies.impl;

import org.slf4j.LoggerFactory;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.FileInputStream;
import de.hybris.platform.catalog.model.CatalogUnawareMediaModel;
import java.util.Map;
import java.util.HashMap;
import java.util.stream.Stream;
import java.nio.file.Path;
import java.io.IOException;
import de.hybris.platform.servicelayer.media.NoDataAvailableException;
import org.apache.commons.lang.StringUtils;
import java.io.Reader;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.Writer;
import de.hybris.platform.util.CSVWriter;
import java.io.FileWriter;
import java.nio.file.Files;
import java.nio.file.attribute.FileAttribute;
import com.bpcl.lpg.core.model.LPGConnectionExportCronjobModel;
import de.hybris.platform.core.model.media.MediaModel;
import javax.annotation.Resource;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.media.MediaService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.strategies.LPGConnectionCSVWriterStrategy;

public class DefaultLPGConnectionCSVWriterStrategy implements LPGConnectionCSVWriterStrategy
{
    private static final Logger LOG;
    private static final String CONSUMER_TYPE_COMMERCIAL = "COMMERCIAL";
    private static final String COMMERCIAL_MEDIA_UID = "lpgCommercialConnectionCSV";
    private static final String DOMESTIC_MEDIA_UID = "bpclCustomerCSV";
    private static final String CONSUMER_TYPE_DOMESTIC = "Domestic";
    private String delimiter;
    private Long numberOfLinesToSkip;
    private Integer attributesLength;
    private MediaService mediaService;
    private Integer uidIndex;
    private Integer lpgIdIndex;
    private Integer consumerIDIndex;
    private Integer consumerNumberIndex;
    private Integer consumerStatusIndex;
    private Integer connectionTypeIndex;
    private Integer distIdIndex;
    private Integer lpgpaymentInfosIndex;
    private Integer codRefillsIndex;
    private Integer consumerTypeIndex;
    private ModelService modelService;
    @Resource(name = "configurationService")
    private ConfigurationService configurationService;
    
    public boolean processCustomerCSVAndCreateLPGConnectionCSV(final MediaModel customerCSVMediaModel, final LPGConnectionExportCronjobModel model) {
        String absoluteCustomerOne = null;
        FileWriter csvWriterCustomerOne = null;
        String absoluteCustomerTwo = null;
        FileWriter csvWriterCustomerTwo = null;
        String absoluteCustomerThree = null;
        FileWriter csvWriterCustomerThree = null;
        String absoluteCustomerFour = null;
        FileWriter csvWriterCustomerFour = null;
        String absoluteCustomerFive = null;
        FileWriter csvWriterCustomerFive = null;
        String absoluteCustomerSix = null;
        FileWriter csvWriterCustomerSix = null;
        String absoluteCustomerSeven = null;
        FileWriter csvWriterCustomerSeven = null;
        String absoluteCustomerEight = null;
        FileWriter csvWriterCustomerEight = null;
        String absoluteCustomerNine = null;
        FileWriter csvWriterCustomerNine = null;
        String absoluteCustomerTen = null;
        FileWriter csvWriterCustomerTen = null;
        final String maxLimit = this.configurationService.getConfiguration().getString("csvsplitter.max.limit");
        final long maxLimitSize = Long.parseLong(maxLimit);
        try {
            final Path pathCustomerOne = Files.createTempFile("bpclCustomerCSVOne", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerOne = pathCustomerOne.toAbsolutePath().toString();
            csvWriterCustomerOne = new FileWriter(pathCustomerOne.toFile());
            final CSVWriter customerOne = new CSVWriter((Writer)csvWriterCustomerOne);
            final Path pathCustomerTwo = Files.createTempFile("bpclCustomerCSVTwo", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerTwo = pathCustomerTwo.toAbsolutePath().toString();
            csvWriterCustomerTwo = new FileWriter(pathCustomerTwo.toFile());
            final CSVWriter customerTwo = new CSVWriter((Writer)csvWriterCustomerTwo);
            final Path pathCustomerThree = Files.createTempFile("bpclCustomerCSVThree", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerThree = pathCustomerThree.toAbsolutePath().toString();
            csvWriterCustomerThree = new FileWriter(pathCustomerThree.toFile());
            final CSVWriter customerThree = new CSVWriter((Writer)csvWriterCustomerThree);
            final Path pathCustomerFour = Files.createTempFile("bpclCustomerCSVFour", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerFour = pathCustomerFour.toAbsolutePath().toString();
            csvWriterCustomerFour = new FileWriter(pathCustomerFour.toFile());
            final CSVWriter customerFour = new CSVWriter((Writer)csvWriterCustomerFour);
            final Path pathCustomerFive = Files.createTempFile("bpclCustomerCSVFive", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerFive = pathCustomerFive.toAbsolutePath().toString();
            csvWriterCustomerFive = new FileWriter(pathCustomerFive.toFile());
            final CSVWriter customerFive = new CSVWriter((Writer)csvWriterCustomerFive);
            final Path pathCustomerSix = Files.createTempFile("bpclCustomerCSVSix", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerSix = pathCustomerSix.toAbsolutePath().toString();
            csvWriterCustomerSix = new FileWriter(pathCustomerSix.toFile());
            final CSVWriter customerSix = new CSVWriter((Writer)csvWriterCustomerSix);
            final Path pathCustomerSeven = Files.createTempFile("bpclCustomerCSVSeven", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerSeven = pathCustomerSeven.toAbsolutePath().toString();
            csvWriterCustomerSeven = new FileWriter(pathCustomerSeven.toFile());
            final CSVWriter customerSeven = new CSVWriter((Writer)csvWriterCustomerSeven);
            final Path pathCustomerEight = Files.createTempFile("bpclCustomerCSVEight", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerEight = pathCustomerEight.toAbsolutePath().toString();
            csvWriterCustomerEight = new FileWriter(pathCustomerEight.toFile());
            final CSVWriter customerEight = new CSVWriter((Writer)csvWriterCustomerEight);
            final Path pathCustomerNine = Files.createTempFile("bpclCustomerCSVNine", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerNine = pathCustomerNine.toAbsolutePath().toString();
            csvWriterCustomerNine = new FileWriter(pathCustomerNine.toFile());
            final CSVWriter customerNine = new CSVWriter((Writer)csvWriterCustomerNine);
            final Path pathCustomerTen = Files.createTempFile("bpclCustomerCSVTen", ".csv", (FileAttribute<?>[])new FileAttribute[0]);
            absoluteCustomerTen = pathCustomerTen.toAbsolutePath().toString();
            csvWriterCustomerTen = new FileWriter(pathCustomerTen.toFile());
            final CSVWriter customerTen = new CSVWriter((Writer)csvWriterCustomerTen);
            final BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> lines = bufferedReader.lines();
            final Stream<String> linesOne = lines.limit(maxLimitSize);
            final BufferedReader bufferedReaderTwo = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> linesbfrtwo = bufferedReaderTwo.lines();
            final Stream<String> linesTwo = linesbfrtwo.skip(maxLimitSize * 1L).limit(maxLimitSize);
            final BufferedReader bufferedReaderThree = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> linesbfrThree = bufferedReaderThree.lines();
            final Stream<String> linesThree = linesbfrThree.skip(maxLimitSize * 2L).limit(maxLimitSize);
            final BufferedReader bufferedReaderFour = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> linesbfrFour = bufferedReaderFour.lines();
            final Stream<String> linesFour = linesbfrFour.skip(maxLimitSize * 3L).limit(maxLimitSize);
            final BufferedReader bufferedReaderFive = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> linesbfrFive = bufferedReaderFive.lines();
            final Stream<String> linesFive = linesbfrFive.skip(maxLimitSize * 4L).limit(maxLimitSize);
            final BufferedReader bufferedReaderSix = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> linesbfrSix = bufferedReaderSix.lines();
            final Stream<String> linesSix = linesbfrSix.skip(maxLimitSize * 5L).limit(maxLimitSize);
            final BufferedReader bufferedReaderSeven = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> linesbfrSeven = bufferedReaderSeven.lines();
            final Stream<String> linesSeven = linesbfrSeven.skip(maxLimitSize * 6L).limit(maxLimitSize);
            final BufferedReader bufferedReaderEight = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> linesbfrEight = bufferedReaderEight.lines();
            final Stream<String> linesEight = linesbfrEight.skip(maxLimitSize * 7L).limit(maxLimitSize);
            final BufferedReader bufferedReaderNine = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> linesbfrNine = bufferedReaderNine.lines();
            final Stream<String> linesNine = linesbfrNine.skip(maxLimitSize * 8L).limit(maxLimitSize);
            final BufferedReader bufferedReaderTen = new BufferedReader(new InputStreamReader(this.getMediaService().getStreamFromMedia(customerCSVMediaModel)));
            final Stream<String> linesbfrTen = bufferedReaderTen.lines();
            final Stream<String> linesTen = linesbfrTen.skip(maxLimitSize * 9L).limit(maxLimitSize);
            linesOne.skip(this.getNumberOfLinesToSkip()).filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerOne));
            customerOne.close();
            linesTwo.filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerTwo));
            customerTwo.close();
            linesThree.filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerThree));
            customerThree.close();
            linesFour.filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerFour));
            customerFour.close();
            linesFive.filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerFive));
            customerFive.close();
            linesSix.filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerSix));
            customerSix.close();
            linesSeven.filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerSeven));
            customerSeven.close();
            linesEight.filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerEight));
            customerEight.close();
            linesNine.filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerNine));
            customerNine.close();
            linesTen.filter(line -> StringUtils.isNotBlank(line)).forEach(line -> this.processRow(line, customerTen));
            customerTen.close();
        }
        catch (final NoDataAvailableException | IllegalArgumentException | IOException e) {
            DefaultLPGConnectionCSVWriterStrategy.LOG.error("Could Not prepare csv file", e);
            return Boolean.FALSE;
        }
        finally {
            if (csvWriterCustomerOne != null) {
                try {
                    csvWriterCustomerOne.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCustomerTwo != null) {
                try {
                    csvWriterCustomerTwo.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCustomerThree != null) {
                try {
                    csvWriterCustomerThree.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCustomerFour != null) {
                try {
                    csvWriterCustomerFour.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCustomerFive != null) {
                try {
                    csvWriterCustomerFive.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCustomerSix != null) {
                try {
                    csvWriterCustomerSix.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCustomerSeven != null) {
                try {
                    csvWriterCustomerSeven.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCustomerEight != null) {
                try {
                    csvWriterCustomerEight.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCustomerNine != null) {
                try {
                    csvWriterCustomerNine.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            if (csvWriterCustomerTen != null) {
                try {
                    csvWriterCustomerTen.close();
                }
                catch (final IOException ioe) {
                    ioe.printStackTrace();
                }
            }
        }
        return Boolean.TRUE;
    }
    
    protected void processRow(final String line, final CSVWriter customerOne) {
        final String[] customerAttributes = line.split(this.delimiter);
        final String customerId = StringUtils.trim(customerAttributes[this.getUidIndex()]);
        DefaultLPGConnectionCSVWriterStrategy.LOG.info("Processing the customer : {} ", customerId);
        this.writeToCSV(customerAttributes, customerOne);
    }
    
    protected void writeToCSV(final String[] customerAttributes, final CSVWriter csv) {
        try {
            int i = 0;
            final Map<Integer, String> line = new HashMap<Integer, String>();
            for (i = 0; i < this.getAttributesLength(); ++i) {
                if (i < customerAttributes.length) {
                    line.put(i, StringUtils.trim(customerAttributes[i]));
                }
                else {
                    line.put(i, "");
                }
            }
            csv.write((Map)line);
        }
        catch (final IOException e) {
            DefaultLPGConnectionCSVWriterStrategy.LOG.error("Exception occurred while writing CSV : " + e);
        }
    }
    
    protected void createLPGConnectionCSVMedia(final LPGConnectionExportCronjobModel cronjobModel, final String absolutePathDomestic, final CatalogUnawareMediaModel lpgDomesticConnectionCSV, final String absolutePathCommercial, final CatalogUnawareMediaModel lpgCommercialConnectionCSV) throws FileNotFoundException {
        this.getModelService().save((Object)lpgDomesticConnectionCSV);
        this.getModelService().save((Object)lpgCommercialConnectionCSV);
        this.getMediaService().setStreamForMedia((MediaModel)lpgDomesticConnectionCSV, (InputStream)new FileInputStream(absolutePathDomestic));
        this.getMediaService().setStreamForMedia((MediaModel)lpgCommercialConnectionCSV, (InputStream)new FileInputStream(absolutePathCommercial));
        this.getModelService().save((Object)lpgDomesticConnectionCSV);
        this.getModelService().save((Object)lpgCommercialConnectionCSV);
        cronjobModel.setLpgDomesticConnectionMedia(lpgDomesticConnectionCSV);
        cronjobModel.setLpgCommercialConnectionMedia(lpgCommercialConnectionCSV);
        this.getModelService().save((Object)cronjobModel);
    }
    
    private CatalogUnawareMediaModel prepareMedia(final String code, final String reportFileName, final String mimeType) {
        final CatalogUnawareMediaModel media = (CatalogUnawareMediaModel)this.modelService.create((Class)CatalogUnawareMediaModel.class);
        media.setCode(code);
        media.setMime(mimeType);
        media.setRealFileName(reportFileName);
        return media;
    }
    
    public String getDelimiter() {
        return this.delimiter;
    }
    
    public void setDelimiter(final String delimiter) {
        this.delimiter = delimiter;
    }
    
    public Long getNumberOfLinesToSkip() {
        return this.numberOfLinesToSkip;
    }
    
    public void setNumberOfLinesToSkip(final Long numberOfLinesToSkip) {
        this.numberOfLinesToSkip = numberOfLinesToSkip;
    }
    
    public Integer getAttributesLength() {
        return this.attributesLength;
    }
    
    public void setAttributesLength(final Integer attributesLength) {
        this.attributesLength = attributesLength;
    }
    
    public MediaService getMediaService() {
        return this.mediaService;
    }
    
    public void setMediaService(final MediaService mediaService) {
        this.mediaService = mediaService;
    }
    
    public Integer getUidIndex() {
        return this.uidIndex;
    }
    
    public void setUidIndex(final Integer uidIndex) {
        this.uidIndex = uidIndex;
    }
    
    public Integer getLpgIdIndex() {
        return this.lpgIdIndex;
    }
    
    public void setLpgIdIndex(final Integer lpgIdIndex) {
        this.lpgIdIndex = lpgIdIndex;
    }
    
    public Integer getConsumerIDIndex() {
        return this.consumerIDIndex;
    }
    
    public void setConsumerIDIndex(final Integer consumerIDIndex) {
        this.consumerIDIndex = consumerIDIndex;
    }
    
    public Integer getConsumerNumberIndex() {
        return this.consumerNumberIndex;
    }
    
    public void setConsumerNumberIndex(final Integer consumerNumberIndex) {
        this.consumerNumberIndex = consumerNumberIndex;
    }
    
    public Integer getConsumerStatusIndex() {
        return this.consumerStatusIndex;
    }
    
    public void setConsumerStatusIndex(final Integer consumerStatusIndex) {
        this.consumerStatusIndex = consumerStatusIndex;
    }
    
    public Integer getConnectionTypeIndex() {
        return this.connectionTypeIndex;
    }
    
    public void setConnectionTypeIndex(final Integer connectionTypeIndex) {
        this.connectionTypeIndex = connectionTypeIndex;
    }
    
    public Integer getDistIdIndex() {
        return this.distIdIndex;
    }
    
    public void setDistIdIndex(final Integer distIdIndex) {
        this.distIdIndex = distIdIndex;
    }
    
    public Integer getLpgpaymentInfosIndex() {
        return this.lpgpaymentInfosIndex;
    }
    
    public void setLpgpaymentInfosIndex(final Integer lpgpaymentInfosIndex) {
        this.lpgpaymentInfosIndex = lpgpaymentInfosIndex;
    }
    
    public Integer getCodRefillsIndex() {
        return this.codRefillsIndex;
    }
    
    public void setCodRefillsIndex(final Integer codRefillsIndex) {
        this.codRefillsIndex = codRefillsIndex;
    }
    
    public Integer getConsumerTypeIndex() {
        return this.consumerTypeIndex;
    }
    
    public void setConsumerTypeIndex(final Integer consumerTypeIndex) {
        this.consumerTypeIndex = consumerTypeIndex;
    }
    
    public ModelService getModelService() {
        return this.modelService;
    }
    
    public void setModelService(final ModelService modelService) {
        this.modelService = modelService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(DefaultLPGConnectionCSVWriterStrategy.class);
    }
}
